@@ -785,6 +785,7 @@ sub WriteQuickTime($$$)
     $et or return 1;    # allow dummy access to autoload this package
     my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $delCount);
     my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err, $atomCount);
+    my ($tag, $lastTag, $errStr);
     my $outfile = $$dirInfo{OutFile} || return 0;
     my $raf = $$dirInfo{RAF};       # (will be null for lower-level atoms)
     my $dataPt = $$dirInfo{DataPt}; # (will be null for top-level atoms)
@@ -857,7 +858,10 @@ sub WriteQuickTime($$$)
     }
     $atomCount = $$tagTablePtr{VARS}{ATOM_COUNT} if $$tagTablePtr{VARS};
 
+    $tag = $lastTag = '';
+
     for (;;) {      # loop through all atoms at this level
+        $lastTag = $tag if $$tagTablePtr{$tag};    # keep track of last known tag
         if (defined $atomCount and --$atomCount < 0 and $dataPt) {
             # stop processing now and just copy the rest of the atom
             Write($outfile, substr($$dataPt, $raf->Tell())) or $rtnVal=$rtnErr, $err=1;
@@ -876,15 +880,15 @@ sub WriteQuickTime($$$)
             last;
         }
         my $size = Get32u(\$hdr, 0) - 8;    # (atom size without 8-byte header)
-        my $tag = substr($hdr, 4, 4);
+        $tag = substr($hdr, 4, 4);
         if ($size == -7) {
             # read the extended size
-            $raf->Read($buff, 8) == 8 or $et->Error('Truncated extended atom'), last;
+            $raf->Read($buff, 8) == 8 or $errStr = 'Truncated extended atom', last;
             $hdr .= $buff;
             my ($hi, $lo) = unpack('NN', $buff);
             if ($hi or $lo > 0x7fffffff) {
                 if ($hi > 0x7fffffff) {
-                    $et->Error('Invalid atom size');
+                    $errStr = 'Invalid atom size';
                     last;
                 } elsif (not $et->Options('LargeFileSupport')) {
                     $et->Error('End of processing at large atom (LargeFileSupport not enabled)');
@@ -892,7 +896,7 @@ sub WriteQuickTime($$$)
                 }
             }
             $size = $hi * 4294967296 + $lo - 16;
-            $size < 0 and $et->Error('Invalid extended atom size'), last;
+            $size < 0 and $errStr = 'Invalid extended atom size', last;
         } elsif ($size == -8) {
             if ($dataPt) {
                 last if $$dirInfo{DirName} eq 'CanonCNTH';  # (this is normal for Canon CNTH atom)
@@ -908,7 +912,7 @@ sub WriteQuickTime($$$)
             }
             last;
         } elsif ($size < 0) {
-            $et->Error('Invalid atom size');
+            $errStr = 'Invalid atom size';
             last;
         }
 
@@ -952,11 +956,11 @@ sub WriteQuickTime($$$)
                 $tag = PrintableTagID($tag,3);
                 if ($size > $maxReadLen and $got == 0x10000) {
                     my $mb = int($size / 0x100000 + 0.5);
-                    $et->Error("'${tag}' atom is too large for rewriting ($mb MB)");
+                    $errStr = "'${tag}' atom is too large for rewriting ($mb MB)";
                 } else {
-                    $et->Error("Truncated '${tag}' atom");
+                    $errStr = "Truncated '${tag}' atom";
                 }
-                return $rtnVal;
+                last;
             }
         }
         # save the handler type for this track
@@ -1446,6 +1450,22 @@ sub WriteQuickTime($$$)
             Write($outfile, $hdr, $buff) or $rtnVal=$rtnErr, $err=1, last;
         }
     }
+    # ($errStr is set if there was an error that could possibly be due to an unknown trailer)
+    if ($errStr) {
+        if ($lastTag eq 'mdat' and not $dataPt and not $$tagTablePtr{$tag}) {
+            my $nvTrail = $et->GetNewValueHash($Image::ExifTool::Extra{Trailer});
+            if ($$et{DEL_GROUP}{Trailer} or ($nvTrail and not ($$nvTrail{Value} and $$nvTrail{Value}[0]))) {
+                $errStr =~ s/ is too large.*//;
+                $et->Warn('Deleted unknown trailer with ' . lcfirst($errStr));
+            } else {
+                $et->Warn('Unknown trailer with ' . lcfirst($errStr));
+                $et->Error('Use "-trailer=" to delete unknown trailer');
+            }
+        } else {
+            $et->Error($errStr);
+            return $dataPt ? undef : 1;
+        }
+    }
     $et->VPrint(0, "  [deleting $delCount $dirName tag".($delCount==1 ? '' : 's')."]\n") if $delCount;
 
     $createKeys &= ~0x01 unless $$addDirs{Keys};   # (Keys may have been written)
